# git-dist-rpm.repo.mk
#
# This makefile template is part of emp-makefile-templates
#
# To use it copy it to your project, and include it there. Do not include
# it directly, cause then historic version won't build on template changes.
# For easy updating do not change your template copy.
#
# Required make variables:
#  NAME, VERSION, DISTDIR, DISTFILE, RPMS, SRPMS, REPO, SRCREPO
# Required targets in including Makefile:
#  $(DISTFILE) custom-check custom-clean

.PHONY: dist
dist:	check $(DISTFILE) ## create distributable archive

.PHONY: rpm
rpm:	check $(RPMS) $(SRPM) ## create rpm packages

.PHONY: srpm
srpm:	check $(RPMS) $(SRPM) ## create only srpm package

$(SRPM): $(DISTFILE)
	rpmbuild -ts --define "_srcrpmdir $(PWD)/rpmbuild" $<
	mv rpmbuild/$(SRPM) .

$(RPMS): $(DISTFILE)
	rpmbuild -tb --define "_rpmdir $(PWD)/rpmbuild" $<
	mv rpmbuild/*/* rpmbuild/.
	for f in $(RPMS); do mv "rpmbuild/$$f" .; done

.PHONY: repo
repo:   check check-if-version-in-changelog is-git-status-clean $(SRPM) $(RPMS) ## push rpms to repository
	scp $(RPMS) $(REPO)
	scp $(SRPM) $(SRCREPO)
	@echo new version pushed to repo, tagging git repo
	make tag

.PHONY: check
check: custom-check check-for-self-update ## run code checks
	@rpmlint --file /usr/share/emp-makefile-templates/rpmlint.config \
	    $(NAME).spec

.PHONY: clean
clean: custom-clean ## clean up
	find . -name \*~ -print0 | \
		xargs -0 --no-run-if-empty rm -- # remove backup files
	rm -rf -- $(DISTDIR) $(DISTFILE) $(RPMS) $(SRPM) rpmbuild

.PHONY: tag
tag: is-git-status-clean ## git tag v$(shell cat VERSION)
	git tag v$(shell cat VERSION)

.PHONY: push
push: ## git push && git push --tags
	git push && git push --tags

.PHONY: is-git-status-clean
is-git-status-clean: clean ## this target fails if git status is not empty
	[[ -z "$(shell git status --porcelain)" ]] || { \
		echo "is-git-status-clean: git status is not empty:" >&2; \
		git status; \
		false; \
	}

# we clean here, so no files stay alive that won't match new VERSION
.PHONY: bump-version
bump-version: clean ## clean and increase content of VERSION by 1
	@OLDV=$$(cat VERSION); \
	sed -ri 's#^(.*\.)([[:digit:]]+)$$#echo "\1$$((\2 + 1))"#e' VERSION; \
	echo "bumped version from $$OLDV to $$(cat VERSION)"

# we clean here, so no files stay alive that won't match new VERSION
.PHONY: edit-version
edit-version: clean ## clean and open VERSION in $EDITOR or vi
	@if [[ -z "$$EDITOR" ]]; \
	then \
		vi "$$EDITOR"; \
	else \
		vi VERSION; \
	fi

# check if the version in file VERSION matches the latest CHANGELOG.md entry
.PHONY: check-if-version-in-changelog
check-if-version-in-changelog: ## check if $(cat VERSION) is in CHANGELOG.md
	@if [ -f CHANGELOG.md ]; \
	then \
		if [ "$(shell sed -r 's=^##[[:space:]]+\[([[:alnum:]]+(\.[[:alnum:]]+)*)\][[:space:]]+\-.*$$=\1=p;d' < CHANGELOG.md | head -1)" != "$(shell cat VERSION)" ]; \
		then echo "Aborted: Current VERSION not first in changelog!"; \
		fi; \
	fi

.PHONY:
help: ## display this help
	@awk 'BEGIN {FS = ":.*##"; printf "Usage:\n  make \033[36m<target>\033[0m\n\nTargets:\n"} /^[a-zA-Z_-]+:.*?##/ { printf "  \033[36m%-10s\033[0m %s\n", $$1, $$2 }' $(MAKEFILE_LIST)

.PHONY: check-for-self-update
check-for-self-update:
	@if ! cmp -s $(lastword $(MAKEFILE_LIST)) \
	    /usr/share/emp-makefile-templates/git-dist-rpm-repo.mk; \
	then \
	    echo "WARNING: $(lastword $(MAKEFILE_LIST)) differs from" \
	        "/usr/share/emp-makefile-templates/git-dist-rpm-repo.mk"; \
	fi

